## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
##
## 12monkeys "Modified Special Domestic Advisor" 
##
## Based on "Special Domestic Advisor" from Requies (civfanatics)
## 
## Version 1.1 / 24.12.05 
## requires Civ4 patch 1.52
##

from CvPythonExtensions import *
# Needed for DomPyCity which inherits from PyCity
# (subclassed for additional functions)
import DomPyHelpers
import PyHelpers
import CvUtil
import ScreenInput
import CvScreenEnums

# Needed to check for non-numbers (specially search function)
import re

PyPlayer = DomPyHelpers.DomPyPlayer
PyCity = DomPyHelpers.DomPyCity

#	IMPORTANT INFORMATION
#	
#	All widget names MUST be unique when creating screens.  If you create
#	a widget named 'Hello', and then try to create another named 'Hello', it
#	will modify the first hello.
#
#	Also, when attaching widgets, 'Background' is a reserve word meant for
#	the background widget.  Do NOT use 'Background' to name any widget, but
#	when attaching to the background, please use the 'Background' keyword.

# globals
gc = CyGlobalContext()
ArtFileMgr = CyArtFileMgr()
localText = CyTranslator()

# Debugging help
def debug (stuff):
	stuff = "SpecDomAdv: " + stuff
	CyInterface().addImmediateMessage(stuff,"")



class CvModSpecialDomesticAdvisor:
	"Modified Special Domestic Advisor Screen"

	# Called when cIV is first booted up and everytime you switch out of cIV
	def __init__(self):
		self.NO_CITY = -1

		self.nTableX = 10   # Right/Left Margin
		self.nTableY = 70	# topmargin without header col (strange behavior of the table, btw)
		
		self.nPlusOffsetX = -4
		self.nMinusOffsetX = 16
		self.nPlusOffsetY = self.nMinusOffsetY = 30
		self.nPlusWidth = self.nPlusHeight = self.nMinusWidth = self.nMinusHeight = 20

		self.nSpecTextOffsetX = 0
		self.nSpecTextOffsetY = 50
		
		self.SCREEN_NAME = "DomesticAdvisor"
		self.EXIT_NAME = "DomesticExit"
		self.VIEW_NAME = "DomesticView"
		self.REDRAW_NAME = "DomesticRedraw"
		self.DEFAULT_AREA_NAME = "DefaultListBackground"
		self.SPECIAL_AREA_NAME = "SpecialListBackground"
		self.BUILD_AREA_NAME = "BuildListBackground"
		self.WW_AREA_NAME = "WWListBackground"
		self.NW_AREA_NAME = "NWListBackground"
		self.BUILD_NAME = "DomesticBuild"
		self.WW_NAME = "DomesticWW"
		self.NW_NAME = "DomesticNW"
		self.CITY_NAME = "DomesticCity"
		self.SPEC_NAME = "DomesticSpec"
		self.BACKGROUND_ID = "DomesticAdvisorBG"

		self.CULTURE_TEXT_NAME = "DomCultureText"
		self.GP_TEXT_NAME = "DomGPText"
		self.NUMBER_TEXT = "NUM"

		self.SPECIALIST_IMAGE_NAME = "DomCitizenImage"
		self.SPECIALIST_PLUS_NAME = "DomIncreaseSpecialist"
		self.SPECIALIST_MINUS_NAME = "DomDecreaseSpecialist"
		self.SPECIALIST_TEXT_NAME = "DomSpecialistText"

		self.BUILD_BUTTON_NAME = "DomBuildButtonName"
		self.BUILD_BUTTON_LEFT = "DomBuildButtonLeftName"
		self.BUILD_BUTTON_RIGHT = "DomBuildButtonRightName"
		
		self.m_nChosenCity = self.NO_CITY
		self.m_szMode = self.DEFAULT_AREA_NAME
		self.m_bSpecialistChange = False
		self.m_nSpecialistChangeNumber = 0
		self.m_nLastSpecialistChange = 0

		# Default Sorting information (also, column numbering)
		self.DEF_COLUMN_DICT = {
			"AUTOMATION": 0,
			"NAME" : 1,
			"LANDMARKS" : 2,
			"POPULATION" : 3,
			"GARRISON" : 4,
			"HAPPY" : 5,
			"HEALTH" : 6,
			"GROWTH" : 7,
			"FOOD" : 8,
			"PRODUCTION" : 9,
			"MAINTENANCE" : 10,
			"TRADE" : 11,
			"COMMERCE" : 12,
			"GOLD" : 13,
			"RESEARCH" : 14,
			"CULTURE_RATE" : 15,
			"CULTURE" : 16,
			"GREATPEOPLE_RATE" : 17,
			"GREATPEOPLE" : 18,
			"PRODUCING" : 19,
			"TURNS" : 20,
    		}			
			
			
		# Default Column Size Information (The total of all should add up to 1000)
		self.DEF_COLUMN_SIZE_DICT = {
			"AUTOMATION" : 80.0,
			"NAME" : 100.0,
			"LANDMARKS" : 80.0,
			"POPULATION" : 30.0,
			"GARRISON" : 30.0,
			"HAPPY" : 30.0,
			"HEALTH" : 30.0,
			"GROWTH" : 30.0,
			"FOOD" : 30.0,
			"PRODUCTION" : 40.0,
			"MAINTENANCE" : 30.0,
			"TRADE" : 30.0,
			"COMMERCE" : 40.0,
			"GOLD" : 40.0,
			"RESEARCH" : 40.0,
			"CULTURE_RATE" : 40.0,
			"CULTURE" : 50.0,
			"GREATPEOPLE_RATE" : 40.0,
			"GREATPEOPLE" : 40.0,
			"PRODUCING" : 130.0,	
			"TURNS" : 40.0,
			}

		# Default Sorting information (also, column numbering)
		# NOTE: NAME MUST be Column #1 in order to allow a selection to give the
		# Production management screen at the bottom.
		self.SPEC_COLUMN_DICT = {
			"RELIGIONS" : 0,
			"NAME" : 1,
			"SPECIALISTS" : 2,
			"HAPPY" : 3,
			"GROWTH" : 4,
			"FOOD" : 5,
			"PRODUCTION" : 6,
			"GOLD" : 7,
			"RESEARCH" : 8,
			"CULTURE_RATE" : 9,
			"CULTURE" : 10,
			"GREATPEOPLE_RATE" : 11,
			"GREATPEOPLE" : 12,
			"PRODUCING" : 13,
			"TURNS" : 14,
			}

		# Default Column Size Information (The total of all should add up to 1000)
		self.SPEC_COLUMN_SIZE_DICT = {
			"RELIGIONS" : 100.0,
			"NAME" : 120.0,
			"SPECIALISTS" : 215.0,
			"HAPPY" : 30.0,
			"GROWTH" : 35.0,
			"FOOD" : 35.0,
			"PRODUCTION" : 40.0,
			"GOLD" : 40.0,
			"RESEARCH" : 40.0,
			"CULTURE_RATE" : 40.0,
			"CULTURE" : 50.0,
			"GREATPEOPLE_RATE" : 40.0,
			"GREATPEOPLE" : 40.0,
			"PRODUCING" : 130.0,
			"TURNS" : 40.0,
			}
				

		# Values to check to see if we need to color the number
		self.PROBLEM_VALUES_DICT = {
			"GARRISON" : 0,
			"HAPPY" : -1,
			"HEALTH" : -1,
			"GROWTH" : -1,
			"FOOD" : -1,
			}
			
		# Values to check to see if we need to color the number as neutral
		self.NEUTRAL_VALUES_DICT = {
			"HAPPY" : 0,
			"HEALTH" : 0,
			"GROWTH" : 0,
			"FOOD" : 0,
			}

		# Values to check to see if we need to color the number as great
		self.GREAT_VALUES_DICT = {
			"HAPPY" : 6,
			"HEALTH" : 6,
			"FOOD" : 8,
			}

		# Dictionary of the coloring dictionaries!
		self.COLOR_DICT_DICT = {
			"PROBLEM": self.PROBLEM_VALUES_DICT,
			"NEUTRAL": self.NEUTRAL_VALUES_DICT,
			"GREAT": self.GREAT_VALUES_DICT,			
			}

		# This creates the set of ALL coloring keys.
		# Do NOT touch.
		self.COLOR_SET = set()
		for clDict in self.COLOR_DICT_DICT.values():
			self.COLOR_SET.update (clDict.keys())
#		SpecDomPrint (str (self.COLOR_SET))

		# Values to change on an update
		# (True indicates update when we DON'T switch to/from food production)
		# Most of this MIGHT change because the computer might be switching
		# plots it's working or creating/removing specialists.

		# TODO: Unfortunately, right now, we have no way of knowing when it's
		# switching from food production to non-food production and vice versa.
		self.UPDATE_DICT = {
			"HAPPY": False,
			"GROWTH" : False,
			"FOOD": False,
			"PRODUCTION": True,
			"COMMERCE": False,
			"GOLD": False,
			"RESEARCH": False,
			"CULTURE_RATE": False,
			"GREATPEOPLE_RATE": False,
			"PRODUCING": True,
			"SPECIALISTS": False,
			"AUTOMATION": True
			}

		self.HEADER_DICT = None
		self.JUSTIFY_DICT = None
		self.SPECIALIST_ICON_DICT = None
		self.AUTOMATION_ICON_DICT = None
		self.COLOR_DICT = None
		self.CITY_PRODUCING_DICT = {}
		
		# Input handling functions
		self.DomesticScreenInputMap = {
			self.SPEC_NAME				: self.Spec,
			self.SPECIALIST_PLUS_NAME 	: self.HandleSpecialistPlus,
			self.SPECIALIST_MINUS_NAME 	: self.HandleSpecialistMinus,
			self.EXIT_NAME				: self.DomesticExit,
			self.VIEW_NAME				: self.View,
			self.REDRAW_NAME			: self.Redraw,
			self.BUILD_NAME				: self.Build,
			self.WW_NAME				: self.WW,
			self.NW_NAME				: self.NW,
			self.BUILD_BUTTON_LEFT		: self.BuildLeft,
			self.BUILD_BUTTON_RIGHT		: self.BuildRight,
			self.CITY_NAME				: self.City,		
			}
			
		self.lBuildings = None
		self.lWW = None
		self.lNW = None

	def getScreen(self):
		return CyGInterfaceScreen( "DomesticAdvisor", CvScreenEnums.DOMESTIC_ADVISOR )
		
	def SetConstants (self):
		
		screen = self.getScreen()
	
		## screen sizes
		#self.nScreenWidth = 1380
		#self.nScreenLength = 950
		self.nScreenWidth = screen.getXResolution() - 20
		self.nScreenLength = screen.getYResolution() -250

		## table sizes
		self.nTableWidth = self.nScreenWidth - (2 * self.nTableX)
		self.nTableLength = self.nScreenLength - 250

		## culture level legend
		self.nCultureLevelX = self.nScreenWidth - 350			# x-coord of text legend
		self.nCultureLevelY = self.nScreenLength - 150			# y-coord of text legend
		self.nCultureLevelDistance = 13		# y spacing of texts	
		self.nCultureLevelTextOffset = 150	# x spacing between level texts and level points

		## great person 
		self.nGPLevelX = self.nScreenWidth - 90	# x-coord 
		self.nGPLevelY = self.nScreenLength - 150	# y-coord  
		self.nGPLevelDistance = 26 # x spacing between  texts and points
		
		## specialists area in specialist view
		self.nFirstSpecialistX = 30		# x-coord 
		self.nSpecialistY = self.nScreenLength - 150			# y-coord  
		self.nSpecialistWidth = 32
		self.nSpecialistLength = 32
		self.nSpecialistDistance = 70
		
		## selection button positions
		self.nButtonsX_Exit = self.nScreenWidth - 50
		self.nButtonsX_View = self.nScreenWidth - 150
		self.nButtonsX_Redraw = self.nScreenWidth - 250
		self.nButtonsX_City = self.nScreenWidth - 350
		self.nButtonsX_Spec = self.nScreenWidth - 450
		self.nButtonsX_Build = self.nScreenWidth - 550
		self.nButtonsX_WW = self.nScreenWidth - 650
		self.nButtonsX_NW = self.nScreenWidth - 750
		self.nButtonsY = self.nScreenLength - 35

		## column sizes for building/wonder screen
		self.BUILD_DATE_COL_SIZE = int(0.0/1000*self.nTableWidth)  ## invisible column -> size = 0.0
		self.BUILD_NAME_COL_SIZE = int(140.0/1000*self.nTableWidth)

		## building / wonder buttons (pedia buttons) size and position		
		## number of pedia buttons and table columns per screen
		self.BUILD_BUTTON_SIZE 		= self.nTableY/2.0
		self.BUILD_NUM_COLS 		= int((self.nTableWidth - 15 - self.BUILD_DATE_COL_SIZE - self.BUILD_NAME_COL_SIZE ) / self.BUILD_BUTTON_SIZE )
		
		## x-step of pedia button
		self.BUILD_BUTTON_X_STEP 	= int(self.BUILD_BUTTON_SIZE)
		
		## size of pedia buttons
		self.BUILD_BUTTON_X_SIZE 	= int(self.BUILD_BUTTON_SIZE-2)
		self.BUILD_BUTTON_Y_SIZE 	= self.BUILD_BUTTON_X_SIZE
		
		## position of pedia buttons
		self.BUILD_BUTTON_X_OFFSET 	= int(self.nTableX+self.BUILD_DATE_COL_SIZE+self.BUILD_NAME_COL_SIZE)
		self.BUILD_BUTTON_Y_OFFSET 	= int(self.BUILD_BUTTON_SIZE)  #int(self.nTableY-10-self.BUILD_BUTTON_SIZE)
		
		## building / wonder left/right button data
		self.BUILD_BUTTON_LEFT_X 	= 40
		self.BUILD_BUTTON_RIGHT_X 	= 80
		self.BUILD_BUTTON_LR_Y_OFFSET = self.BUILD_BUTTON_Y_OFFSET
		self.BUILD_BUTTON_LR_X_SIZE = 40
		self.BUILD_BUTTON_LR_Y_SIZE = 40
		
		self.BUILD_FROM_COL 		= 0
		self.BUILD_TO_COL 			= self.BUILD_FROM_COL + self.BUILD_NUM_COLS - 1
		self.WW_FROM_COL 			= 0
		self.WW_TO_COL 				= self.WW_FROM_COL + self.BUILD_NUM_COLS - 1
		self.NW_FROM_COL 			= 0
		self.NW_TO_COL 				= self.NW_FROM_COL + self.BUILD_NUM_COLS - 1		
		
			
	def getBuildingType(self, iBuilding):
		if (isWorldWonderClass(gc.getBuildingInfo(iBuilding).getBuildingClassType())):
			return (2)			
		if (isTeamWonderClass(gc.getBuildingInfo(iBuilding).getBuildingClassType())):
			return (2)
		if (isNationalWonderClass(gc.getBuildingInfo(iBuilding).getBuildingClassType())):
			return (1)
		# Regular building
		return (0)
		
	def getBuildingSortedList(self, iBuildingType):
		listBuildings = []
		iCount = 0
		for iBuilding in range(gc.getNumBuildingInfos()):
			if (self.getBuildingType(iBuilding) == iBuildingType):
				listBuildings.append(iBuilding)
				iCount += 1		
		listSorted = [(0,0)] * iCount
		iI = 0
		for iBuilding in listBuildings:
			listSorted[iI] = (iBuilding, gc.getBuildingInfo(iBuilding).getDescription())
			iI += 1
		listSorted.sort()
		return listSorted

	def createDictionaries(self):

		self.lBuildings = self.getBuildingSortedList(0) # Usual Buildings
		self.lWW = self.getBuildingSortedList(2) # World and Team Wonders
		self.lNW = self.getBuildingSortedList(1) # National Wonders	

		if self.BUILD_TO_COL > len(self.lBuildings)-1:
			self.BUILD_TO_COL = len(self.lBuildings)-1
		if self.WW_TO_COL > len(self.lWW)-1:
			self.WW_TO_COL = len(self.lWW)-1
		if self.NW_TO_COL > len(self.lNW)-1:
			self.NW_TO_COL = len(self.lNW)-1

		
		if self.HEADER_DICT == None:
			# Header Information (Must be here, because C++ functions aren't
			# available upon startup of CIV)
			self.HEADER_DICT = {
				"DATE" : localText.getText("TXT_KEY_DOMESTIC_ADVISOR_FOUNDED", ()),
				"NAME" : localText.getText("TXT_KEY_DOMESTIC_ADVISOR_NAME", ()),
				"LANDMARKS" : localText.getText("TXT_KEY_WB_BUILDINGS", ()),
				"POPULATION" : localText.getText("TXT_KEY_POPULATION", ()),
				"GARRISON" : u"%c" % CyGame().getSymbolID(FontSymbols.STRENGTH_CHAR),
				"HAPPY" : u"%c" % CyGame().getSymbolID(FontSymbols.HAPPY_CHAR),
				"HEALTH" : u"%c" % CyGame().getSymbolID(FontSymbols.HEALTHY_CHAR),
#				"GROWTH" : u"%c" % gc.getBonusInfo(19).getChar(),
				"GROWTH" : u"%c" % CyGame().getSymbolID(FontSymbols.BAD_FOOD_CHAR),
				"FOOD" : u"%c" %(gc.getYieldInfo(YieldTypes.YIELD_FOOD).getChar()) ,
				"PRODUCTION" : u"%c" %(gc.getYieldInfo(YieldTypes.YIELD_PRODUCTION).getChar()),
				"MAINTENANCE" : u"%c" % CyGame().getSymbolID(FontSymbols.BAD_GOLD_CHAR),
				"TRADE" : u"%c" % CyGame().getSymbolID(FontSymbols.TRADE_CHAR),
				"COMMERCE" : u"%c" %(gc.getYieldInfo(YieldTypes.YIELD_COMMERCE).getChar()),
				"GOLD" : u"%c" %(gc.getCommerceInfo(CommerceTypes.COMMERCE_GOLD).getChar()),
				"RESEARCH" : u"%c" %(gc.getCommerceInfo(CommerceTypes.COMMERCE_RESEARCH).getChar()),
				"CULTURE_RATE" : u"%c" %(gc.getCommerceInfo(CommerceTypes.COMMERCE_CULTURE).getChar()),
				"CULTURE" : localText.getText("TXT_KEY_TOTAL", ()),
				"GREATPEOPLE_RATE" : u"%c" % CyGame().getSymbolID(FontSymbols.GREAT_PEOPLE_CHAR),
				"GREATPEOPLE" : localText.getText("TXT_KEY_TOTAL", ()),
				"PRODUCING" : localText.getText("TXT_KEY_DOMESTIC_ADVISOR_PRODUCING", ()),
				"RELIGIONS" : localText.getText("TXT_KEY_ADVISOR_RELIGION", ()),
				"SPECIALISTS" : localText.getText("TXT_KEY_CONCEPT_SPECIALISTS", ()),
				"AUTOMATION" : localText.getText("TXT_KEY_DOMESTIC_ADVISOR_AUTOMATION", ()),
			    "TURNS" : "Turns",
			}
				
		if self.JUSTIFY_DICT == None:
			# Header Information (Must be here, because C++ functions aren't
			# available upon startup of CIV)
			self.JUSTIFY_DICT = {
				"DATE" : CvUtil.FONT_LEFT_JUSTIFY, 
				"NAME" : CvUtil.FONT_LEFT_JUSTIFY,
				"LANDMARKS" : CvUtil.FONT_LEFT_JUSTIFY,
				"POPULATION" : CvUtil.FONT_RIGHT_JUSTIFY,
				"GARRISON" : CvUtil.FONT_RIGHT_JUSTIFY,
				"HAPPY" : CvUtil.FONT_RIGHT_JUSTIFY,
				"HEALTH" : CvUtil.FONT_RIGHT_JUSTIFY,
				"GROWTH" : CvUtil.FONT_RIGHT_JUSTIFY,
				"FOOD" : CvUtil.FONT_RIGHT_JUSTIFY,
				"PRODUCTION" : CvUtil.FONT_RIGHT_JUSTIFY,
				"MAINTENANCE" : CvUtil.FONT_RIGHT_JUSTIFY,
				"TRADE" : CvUtil.FONT_RIGHT_JUSTIFY,
				"COMMERCE" : CvUtil.FONT_RIGHT_JUSTIFY,
				"GOLD" : CvUtil.FONT_RIGHT_JUSTIFY,
				"RESEARCH" : CvUtil.FONT_RIGHT_JUSTIFY,
				"CULTURE_RATE" : CvUtil.FONT_RIGHT_JUSTIFY,
				"CULTURE" : CvUtil.FONT_RIGHT_JUSTIFY,
				"GREATPEOPLE_RATE" : CvUtil.FONT_RIGHT_JUSTIFY,
				"GREATPEOPLE" : CvUtil.FONT_RIGHT_JUSTIFY,
				"PRODUCING" : CvUtil.FONT_LEFT_JUSTIFY,
				"RELIGIONS" : CvUtil.FONT_LEFT_JUSTIFY,
				"SPECIALISTS" : CvUtil.FONT_LEFT_JUSTIFY,
				"AUTOMATION" : CvUtil.FONT_LEFT_JUSTIFY,
			    "TURNS" : CvUtil.FONT_RIGHT_JUSTIFY,
			}
			
		if self.SPECIALIST_ICON_DICT == None:
			# Header Information (Must be here, because C++ functions aren't
			# available upon startup of CIV)
			self.SPECIALIST_ICON_DICT = {
				0 : u"%c" % CyGame().getSymbolID(FontSymbols.BULLET_CHAR), # Citizen
				1 : u"%c" % CyGame().getSymbolID(FontSymbols.HAPPY_CHAR), # Priest
				2 : u"%c" %(gc.getCommerceInfo(CommerceTypes.COMMERCE_CULTURE).getChar()), # Artist
				3 : u"%c" %(gc.getCommerceInfo(CommerceTypes.COMMERCE_RESEARCH).getChar()), # Scientist
				4 : u"%c" %(gc.getCommerceInfo(CommerceTypes.COMMERCE_GOLD).getChar()), # Merchant
				5 : u"%c" %(gc.getYieldInfo(YieldTypes.YIELD_PRODUCTION).getChar()), # Engineer
				6 : u"%c" %(CyGame().getSymbolID(FontSymbols.STRENGTH_CHAR)), # General
				7 : u"%c" %(gc.getYieldInfo(YieldTypes.YIELD_COMMERCE).getChar()), # Statesman
				}

		if self.AUTOMATION_ICON_DICT == None:
			# Header Information (Must be here, because C++ functions aren't
			# available upon startup of CIV)
			self.AUTOMATION_ICON_DICT = {
				0 : u"%c" %(gc.getYieldInfo(YieldTypes.YIELD_FOOD).getChar()), # Emphasize Food
				1 : u"%c" %(gc.getYieldInfo(YieldTypes.YIELD_PRODUCTION).getChar()), # Emphasize Production
				2 : u"%c" %(gc.getCommerceInfo(CommerceTypes.COMMERCE_GOLD).getChar()), # Emphasize Gold?
				3 : u"%c" %(gc.getCommerceInfo(CommerceTypes.COMMERCE_RESEARCH).getChar()), # Emphasize Research
				4 : u"%c" % CyGame().getSymbolID(FontSymbols.GREAT_PEOPLE_CHAR), # Emphasize GP
				5 : u"%c" % CyGame().getSymbolID(FontSymbols.BAD_FOOD_CHAR) # Emphasize Avoid Growth
				}

		if self.COLOR_DICT == None:
			# Colors to highlight with for each type of number (Must be here,
			#  because C++ functions aren't available upon startup of CIV)
			self.COLOR_DICT = {
				"PROBLEM": gc.getInfoTypeForString("COLOR_RED"),
				"NEUTRAL": gc.getInfoTypeForString("COLOR_YELLOW"),
				"GREAT": gc.getInfoTypeForString("COLOR_GREEN"),
				}

	def createPositions (self, screen):
		self.nScreenX = ( screen.getXResolution() - self.nScreenWidth ) / 2
		self.nScreenY = ( screen.getYResolution() - self.nScreenLength ) / 5  

	# Create a new screen, called DomesticAdvisor, using the file CvSpecialDomesticAdvisor.py for input
	def getScreen(self):
		return CyGInterfaceScreen(self.SCREEN_NAME, CvScreenEnums.DOMESTIC_ADVISOR)

	def getCurrentCity (self):
		if (self.m_nChosenCity != self.NO_CITY):
			iPlayer = PyPlayer(CyGame().getActivePlayer())
			cityList = iPlayer.getCityList()
			return PyCity( CyGame().getActivePlayer(), cityList[self.m_nChosenCity].getID() )
		else:
			return None

	def getNumSpecialistInfos (self):
		return len (self.SPECIALIST_ICON_DICT)

	def getNumEmphasizeInfos (self):
		return len (self.AUTOMATION_ICON_DICT)

	# Screen construction function
	# This is the function that's called whenever F1 is pressed.
	def interfaceScreen(self):
		self.SetConstants()
		self.createDictionaries()

		# This will get the actual Player class into iPlayer
		iPlayer = PyPlayer(CyGame().getActivePlayer())

		screen = self.getScreen()
#		screen.setForcedRedraw (True)
		self.createPositions (screen)
		screen.setDimensions (self.nScreenX, self.nScreenY, self.nScreenWidth, self.nScreenLength)
		screen.showScreen(PopupStates.POPUPSTATE_IMMEDIATE, False)
		self.drawBasicScreen()
		self.drawScreen (self.m_szMode)

	def drawBasicScreen (self):
		screen = self.getScreen()
		# Here we set the background widget and exit button, and we show the screen
		screen.addPanel( self.BACKGROUND_ID, u"", u"", True, False, 0, 29, self.nScreenWidth, self.nScreenLength, PanelStyles.PANEL_STYLE_MAIN )
		# Text Buttons
		screen.setText(self.WW_NAME, "Background", localText.getText("TXT_KEY_DOMESTIC_WORLD_WONDERS_VIEW", ()), CvUtil.FONT_CENTER_JUSTIFY, self.nButtonsX_WW, self.nButtonsY, -0.1, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )
		screen.setText(self.NW_NAME, "Background", localText.getText("TXT_KEY_DOMESTIC_NATIONAL_WONDERS_VIEW", ()), CvUtil.FONT_CENTER_JUSTIFY, self.nButtonsX_NW, self.nButtonsY, -0.1, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )
		screen.setText(self.BUILD_NAME, "Background", localText.getText("TXT_KEY_DOMESTIC_BUILDINGS_VIEW", ()), CvUtil.FONT_CENTER_JUSTIFY, self.nButtonsX_Build, self.nButtonsY, -0.1, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )
		screen.setText(self.EXIT_NAME, "Background", localText.getText("TXT_KEY_PEDIA_SCREEN_EXIT", ()), CvUtil.FONT_CENTER_JUSTIFY, self.nButtonsX_Exit, self.nButtonsY, -0.1, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_CLOSE_SCREEN, -1, -1 )
		screen.setText(self.VIEW_NAME, "Background", localText.getText("TXT_KEY_DOMESTIC_ADVISOR_VIEW", ()), CvUtil.FONT_CENTER_JUSTIFY, self.nButtonsX_View, self.nButtonsY, -0.1, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )
		screen.setText(self.SPEC_NAME, "Background", localText.getText("TXT_KEY_DOMESTIC_SPECIALISTS_VIEW", ()), CvUtil.FONT_CENTER_JUSTIFY, self.nButtonsX_Spec, self.nButtonsY, -0.1, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )
		screen.setText(self.CITY_NAME, "Background", localText.getText("TXT_KEY_DOMESTIC_CITY_VIEW", ()), CvUtil.FONT_CENTER_JUSTIFY, self.nButtonsX_City, self.nButtonsY, -0.1, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )
		screen.setText(self.REDRAW_NAME, "Background", localText.getText("TXT_KEY_DOMESTIC_ADVISOR_REDRAW", ()), CvUtil.FONT_CENTER_JUSTIFY, self.nButtonsX_Redraw, self.nButtonsY, -0.1, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )
		# Specialist Screen Toggle Button
		self.drawCultureLevel()
		self.drawSpecialists()

	def drawCultureLevel (self):
		screen = self.getScreen()
		# Cultural Levels Text
		screen.setText (self.CULTURE_TEXT_NAME, "Background", self.HEADER_DICT["CULTURE_RATE"], CvUtil.FONT_LEFT_JUSTIFY, self.nCultureLevelX, self.nCultureLevelY, -0.1, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)
		nGameSpeed = CyGame().getGameSpeedType()
		iCount = 2
		for i in range (gc.getNumCultureLevelInfos()):
			pCultureLevel = gc.getCultureLevelInfo (i)
			nValue = pCultureLevel.getSpeedThreshold(nGameSpeed)
			if (nValue != 0):
				screen.setText (self.CULTURE_TEXT_NAME + str(i), "Background", "<font=2>" + pCultureLevel.getText() + "</font>", CvUtil.FONT_LEFT_JUSTIFY, self.nCultureLevelX, self.nCultureLevelY + (self.nCultureLevelDistance * iCount), -0.1, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)
				screen.setText (self.CULTURE_TEXT_NAME + self.NUMBER_TEXT + str(i), "Background", "<font=2>" + str(nValue) + "</font>", CvUtil.FONT_RIGHT_JUSTIFY, self.nCultureLevelX + self.nCultureLevelTextOffset, self.nCultureLevelY + (self.nCultureLevelDistance * iCount), -0.1, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)
				iCount += 1
		iPlayer = PyPlayer(CyGame().getActivePlayer())
		# GP Level Text
		screen.setText (self.GP_TEXT_NAME, "Background", self.HEADER_DICT["GREATPEOPLE_RATE"], CvUtil.FONT_RIGHT_JUSTIFY, self.nGPLevelX, self.nGPLevelY, -0.1, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)
		screen.setText (self.GP_TEXT_NAME + self.NUMBER_TEXT, "Background", "<font=2>" + str (iPlayer.greatPeopleThreshold()) + "</font>", CvUtil.FONT_RIGHT_JUSTIFY, self.nGPLevelX, self.nGPLevelY + self.nGPLevelDistance, -0.1, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)
		
	# Draw the specialist and their increase and decrease buttons
	def drawSpecialists (self):
		screen = self.getScreen()
		# Citizen Buttons
		for i in range( self.getNumSpecialistInfos() ):
			if (gc.getSpecialistInfo(i).isVisible()):
				szName = self.SPECIALIST_IMAGE_NAME + str(i)
				screen.setImageButton( szName, gc.getSpecialistInfo(i).getTexture(), self.nFirstSpecialistX + (self.nSpecialistDistance * i), self.nSpecialistY, self.nSpecialistWidth, self.nSpecialistLength, WidgetTypes.WIDGET_CITIZEN, i, -1 )
				screen.hide( szName )
		# Increase Specialists...
		for i in range( self.getNumSpecialistInfos() ):
			if (gc.getSpecialistInfo(i).isVisible()):
				szName = self.SPECIALIST_PLUS_NAME + str(i)
				screen.setButtonGFC( szName, u"", "", self.nFirstSpecialistX + (self.nSpecialistDistance * i) + self.nPlusOffsetX, self.nSpecialistY + self.nPlusOffsetY, self.nPlusWidth, self.nPlusHeight, WidgetTypes.WIDGET_CHANGE_SPECIALIST, i, 1, ButtonStyles.BUTTON_STYLE_CITY_PLUS )
				screen.hide( szName )
		# Decrease specialists
		for i in range( self.getNumSpecialistInfos() ):
			if (gc.getSpecialistInfo(i).isVisible()):
				szName = self.SPECIALIST_MINUS_NAME + str(i)
				screen.setButtonGFC( szName, u"", "", self.nFirstSpecialistX + (self.nSpecialistDistance * i) + self.nMinusOffsetX, self.nSpecialistY + self.nMinusOffsetY, self.nMinusWidth, self.nMinusHeight, WidgetTypes.WIDGET_CHANGE_SPECIALIST, i, -1, ButtonStyles.BUTTON_STYLE_CITY_MINUS )
				screen.hide( szName )
		# Specialist text
		for i in range (self.getNumSpecialistInfos()):
			if (gc.getSpecialistInfo(i).isVisible()):
				szName = self.SPECIALIST_TEXT_NAME + str(i)
				screen.setText (szName, "Background", "", CvUtil.FONT_LEFT_JUSTIFY, self.nFirstSpecialistX + (self.nSpecialistDistance * i) + self.nSpecTextOffsetX, self.nSpecialistY + self.nSpecTextOffsetY, -0.1, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)
				screen.hide (szName)

	def showSpecialists (self):
		screen = self.getScreen()
		self.hideSpecialists()
		pCity = self.getCurrentCity()
		if (pCity):
			nPopulation = pCity.getPopulation()
			nFreeSpecial = pCity.totalFreeSpecialists()
			# For each specialist type
			for i in range( self.getNumSpecialistInfos() ):
				# Show all the specialist images
				szName = self.SPECIALIST_IMAGE_NAME + str(i)
				screen.show( szName )
				szName = self.SPECIALIST_TEXT_NAME + str(i)
				screen.setText (szName, "Background", str (pCity.getSpecialistCount(i)) + "/" + str (pCity.getMaxSpecialistCount(i)) + u" %c" % self.SPECIALIST_ICON_DICT[i], CvUtil.FONT_LEFT_JUSTIFY, self.nFirstSpecialistX + (self.nSpecialistDistance * i) + self.nSpecTextOffsetX, self.nSpecialistY + self.nSpecTextOffsetY, -0.1, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)
				screen.show (szName)
				# If the specialist is valid
				if (pCity.isSpecialistValid(i, 1) and (pCity.getForceSpecialistCount(i) < (nPopulation + nFreeSpecial))):
					szName = self.SPECIALIST_PLUS_NAME + str(i)
					screen.show( szName )
				if (pCity.getSpecialistCount(i) > 0 or pCity.getForceSpecialistCount(i) > 0 ):
					szName = self.SPECIALIST_MINUS_NAME + str(i)
					screen.show( szName )
							
	def hideSpecialists (self):
		screen = self.getScreen()
		# Hide Buttons
		for i in range( self.getNumSpecialistInfos() ):
			if (gc.getSpecialistInfo(i).isVisible()):
				szName = self.SPECIALIST_IMAGE_NAME + str(i)
				screen.hide (szName)
				szName = self.SPECIALIST_PLUS_NAME + str(i)
				screen.hide (szName)
				szName = self.SPECIALIST_MINUS_NAME + str(i)
				screen.hide (szName)
				szName = self.SPECIALIST_TEXT_NAME + str(i)
				screen.hide (szName)

	def hide (self, screen, screenType):
		screen.hide (screenType)
		if (screenType == self.SPECIAL_AREA_NAME):
			self.hideSpecialists()
		elif (screenType == self.BUILD_AREA_NAME):
			self.HideButtons (self.BUILD_AREA_NAME)
		elif (screenType == self.WW_AREA_NAME):
			self.HideButtons (self.WW_AREA_NAME)
		elif (screenType == self.NW_AREA_NAME):
			self.HideButtons (self.NW_AREA_NAME)
	
			
	# Create the Default Domestic Advisor Screen
	def drawScreen (self, szMode):
		screen = self.getScreen()
		if (szMode == self.DEFAULT_AREA_NAME):
			self.m_szMode = self.DEFAULT_AREA_NAME
			self.hide (screen, self.SPECIAL_AREA_NAME)
			self.hide (screen, self.BUILD_AREA_NAME)
			self.hide (screen, self.WW_AREA_NAME)
			self.hide (screen, self.NW_AREA_NAME)
		elif (szMode == self.SPECIAL_AREA_NAME):
			self.m_szMode = self.SPECIAL_AREA_NAME
			self.hide (screen, self.DEFAULT_AREA_NAME)
			self.hide (screen, self.BUILD_AREA_NAME)
			self.hide (screen, self.WW_AREA_NAME)
			self.hide (screen, self.NW_AREA_NAME)
			self.showSpecialists ()
		elif (szMode == self.BUILD_AREA_NAME):
			self.m_szMode = self.BUILD_AREA_NAME
			self.hide (screen, self.SPECIAL_AREA_NAME)
			self.hide (screen, self.DEFAULT_AREA_NAME)
			self.hide (screen, self.WW_AREA_NAME)
			self.hide (screen, self.NW_AREA_NAME)
			self.ShowButtons (szMode)
		elif (szMode == self.WW_AREA_NAME):
			self.m_szMode = self.WW_AREA_NAME
			self.hide (screen, self.SPECIAL_AREA_NAME)
			self.hide (screen, self.DEFAULT_AREA_NAME)
			self.hide (screen, self.BUILD_AREA_NAME)
			self.hide (screen, self.NW_AREA_NAME)
			self.ShowButtons (szMode)
		elif (szMode == self.NW_AREA_NAME):
			self.m_szMode = self.NW_AREA_NAME
			self.hide (screen, self.SPECIAL_AREA_NAME)
			self.hide (screen, self.DEFAULT_AREA_NAME)
			self.hide (screen, self.BUILD_AREA_NAME)
			self.hide (screen, self.WW_AREA_NAME)
			self.ShowButtons (szMode)
		else:
			return
		# Erase the flag?
		CyInterface().setDirty(InterfaceDirtyBits.MiscButtons_DIRTY_BIT, True)
		# Draw the city list...
		self.drawContents (szMode)
		screen.enableSort (szMode)

	def ShowButtons (self, szArea):
		screen = self.getScreen()
		if (szArea == self.BUILD_AREA_NAME):
			lList = self.lBuildings
			iFrom = self.BUILD_FROM_COL
			iTo = self.BUILD_TO_COL
		elif (szArea == self.WW_AREA_NAME):
			lList = self.lWW
			iFrom = self.WW_FROM_COL
			iTo = self.WW_TO_COL
		elif (szArea == self.NW_AREA_NAME):
			lList = self.lNW
			iFrom = self.NW_FROM_COL
			iTo = self.NW_TO_COL
		else:
			return
		i = 0
		for iI in range(len(lList)):
			# set the header
			if (iI >= iFrom) & ( iI <= iTo ):
				szName = self.BUILD_BUTTON_NAME+str(i)
				screen.setImageButton(  szName, gc.getBuildingInfo(lList[iI][0]).getButton(), \
										self.BUILD_BUTTON_X_OFFSET+i*self.BUILD_BUTTON_X_STEP, self.BUILD_BUTTON_Y_OFFSET, self.BUILD_BUTTON_X_SIZE, self.BUILD_BUTTON_Y_SIZE,   \
										WidgetTypes.WIDGET_PEDIA_JUMP_TO_BUILDING, lList[iI][0], -1 )
				i += 1
		# Left Button
		if (iFrom > 0):
			szName = self.BUILD_BUTTON_LEFT
			screen.setButtonGFC( szName, u"", "", \
							     self.BUILD_BUTTON_LEFT_X, self.BUILD_BUTTON_LR_Y_OFFSET, self.BUILD_BUTTON_LR_X_SIZE, self.BUILD_BUTTON_LR_Y_SIZE, \
								 WidgetTypes.WIDGET_GENERAL, 0, 1, ButtonStyles.BUTTON_STYLE_ARROW_LEFT )			
		else:
			szName = self.BUILD_BUTTON_LEFT
			screen.hide (szName)
			
		# right Button
		if (iTo < len(lList)-1):
			szName = self.BUILD_BUTTON_RIGHT
			screen.setButtonGFC( szName, u"", "", \
							     self.BUILD_BUTTON_RIGHT_X, self.BUILD_BUTTON_LR_Y_OFFSET, self.BUILD_BUTTON_LR_X_SIZE, self.BUILD_BUTTON_LR_Y_SIZE, \
								 WidgetTypes.WIDGET_GENERAL, 0, -1, ButtonStyles.BUTTON_STYLE_ARROW_RIGHT )			
		else:
			szName = self.BUILD_BUTTON_RIGHT
			screen.hide (szName)
					
					
	def HideButtons (self, szArea):
		screen = self.getScreen()
		if (szArea == self.BUILD_AREA_NAME):
			lList = self.lBuildings
			iFrom = self.BUILD_FROM_COL
			iTo = self.BUILD_TO_COL
		elif (szArea == self.WW_AREA_NAME):
			lList = self.lWW
			iFrom = self.WW_FROM_COL
			iTo = self.WW_TO_COL
		elif (szArea == self.NW_AREA_NAME):
			lList = self.lNW
			iFrom = self.NW_FROM_COL
			iTo = self.NW_TO_COL
		else:
			return
		i = 0
		for iI in range(len(lList)):
			if (iI >= iFrom) & ( iI <= iTo ):
				szName = self.BUILD_BUTTON_NAME+str(i)
				screen.hide (szName)
				i += 1
		szName = self.BUILD_BUTTON_LEFT
		screen.hide (szName)
		szName = self.BUILD_BUTTON_RIGHT
		screen.hide (szName)
				
		
	# headers...
	def drawHeaders (self, szArea):
		# Get the screen
		screen = self.getScreen()
		if ((szArea == self.BUILD_AREA_NAME) or (szArea == self.WW_AREA_NAME) or (szArea == self.NW_AREA_NAME)):
			screen.setTableColumnHeader (szArea, 0, "<font=2>" + localText.getText("TXT_KEY_DOMESTIC_ADVISOR_FOUNDED", ()) + "</font>", self.BUILD_DATE_COL_SIZE )			
			screen.setTableColumnHeader (szArea, 1, "<font=2>" + localText.getText("TXT_KEY_DOMESTIC_ADVISOR_NAME", ()) + "</font>", self.BUILD_NAME_COL_SIZE )			
			if (szArea == self.BUILD_AREA_NAME):
				lList = self.lBuildings
				iFrom = self.BUILD_FROM_COL
				iTo = self.BUILD_TO_COL
			elif (szArea == self.WW_AREA_NAME):
				lList = self.lWW
				iFrom = self.WW_FROM_COL
				iTo = self.WW_TO_COL
			elif (szArea == self.NW_AREA_NAME):
				lList = self.lNW	
				iFrom = self.NW_FROM_COL
				iTo = self.NW_TO_COL
			else:
				return
			i = 2
			for iI in range(len(lList)):
				# set the header
				if (iI >= iFrom) & ( iI <= iTo ):
					screen.setTableColumnHeader (szArea, i, "<font=2>" + lList[iI][1]+ "</font>", self.BUILD_BUTTON_X_STEP )			
					i += 1
		else:
			if (szArea == self.DEFAULT_AREA_NAME):
				lKVPairs = self.DEF_COLUMN_DICT.items()
				dColSizeDict = self.DEF_COLUMN_SIZE_DICT
			elif (szArea == self.SPECIAL_AREA_NAME):
				lKVPairs = self.SPEC_COLUMN_DICT.items()
				dColSizeDict = self.SPEC_COLUMN_SIZE_DICT
			else:
				return
			# For each column
			for key, value in lKVPairs:
				# set the header
				screen.setTableColumnHeader (szArea, value, "<font=2>" + self.HEADER_DICT[key] + "</font>", int(dColSizeDict[key]/1010*self.nTableWidth) )			

	# Function to calculate value given a certain city and key
	def CalculateValue (self, pCity, szKey):
		# Initialize Return Value
		szReturn = ""

		if (szKey == "DATE"):
			# City founded date...
			szIDText = "Date" + str(pCity.getID())
			iTurnTime = pCity.getGameTurnFounded()
			szReturn = unicode(CyGameTextMgr().getTimeStr(iTurnTime, false))

		elif (szKey == "NAME"):
			# City name...
			szReturn = pCity.getName()

		elif (szKey == "LANDMARKS"):
			# Landmarks...

			# First look for Government Centers
			if pCity.isGovernmentCenter():
				szStar = u"%c" % CyGame().getSymbolID(FontSymbols.STAR_CHAR)
				szSilverStar = u"%c" % CyGame().getSymbolID(FontSymbols.SILVER_STAR_CHAR)
				# And distinguish between the Capital and the others (Forbidden Palace
				# and Versailles)
				if pCity.isCapital():
					szReturn += szStar
				else:
					szReturn += szSilverStar

			# add National Wonder Landmarks
			nWonder = 0
      # hard coded building id strings from xml (not sure if want to keep
			# hard-coded but can't think of an easier way of doing this)
			for nWonder in range (64,75):
				if pCity.hasBuilding(nWonder):
					# Use bullets as markers for National Wonders
					szBullet = u"%c" % CyGame().getSymbolID(FontSymbols.BULLET_CHAR)
					szReturn += szBullet
			# end National Wonder Landmarks

			if pCity.isPower():
				szPower = u"%c" % CyGame().getSymbolID(FontSymbols.POWER_CHAR)
				szReturn += szPower
			if pCity.isDisorder():
				szOccu = u"%c" % CyGame().getSymbolID(FontSymbols.OCCUPATION_CHAR)
				szAngry = u"%c" % CyGame().getSymbolID(FontSymbols.ANGRY_POP_CHAR)

				if pCity.isOccupation():
					szReturn += szOccu +":"+unicode(pCity.getOccupationTimer())
				else:
					szReturn += szAngry

		elif (szKey == "POPULATION"):
			# Population
			szReturn = unicode(pCity.getPopulation())

		elif (szKey == "GARRISON"):
			# Garrison
			szReturn = unicode(pCity.getMilitaryHappinessUnits())

		elif (szKey == "HAPPY"):
			# Replace Happiness/Unhappiness with Net Happiness...
			szReturn = unicode(pCity.getHappyPopulation() - pCity.getUnhappyPopulation())

		elif (szKey == "HEALTH"):
			# combine health/unhealthy... to Net City Health
			szReturn = unicode(pCity.getGoodHealth() - pCity.getBadHealth())

		elif (szKey == "GROWTH"):
			# Turns til Growth
 			nFood = pCity.foodDifference (True)

			# If this is a food production (i.e., worker or settler)
			if (pCity.isFoodProduction()):
				# We need to indicate there's no growth manually
				szReturn = "-"
			else:
				# Otherwise, just call FoodTurnsLeft
				szReturn = unicode (pCity.getFoodTurnsLeft())

			# Not enough food, so calculate how many turns until we starve.
			# We put this here because we still starve if building a food production
			if nFood < 0:
				# Use floor divide (//) because we want the number it'll drop below 0
				# (that's also the reason for the additional 1)
				szReturn = unicode ((pCity.getFood()+1) // nFood)

		elif (szKey == "FOOD"):
			# Replace Food/Food Used with Net Food AND Turns until Growth...

			# If this is a food production (i.e., worker or settler)
			if (pCity.isFoodProduction()):
				nFood = pCity.getCurrentProductionDifference (False, False) - pCity.getCurrentProductionDifference (True, False)
			else:
				nFood = pCity.foodDifference (True)
			szReturn = unicode(nFood)

		elif (szKey == "PRODUCTION"):
			# Production status
			szReturn = unicode(pCity.getCurrentProductionDifference (True, False))

		elif (szKey == "MAINTENANCE"):
			# City Maintanance...
			szReturn = unicode(pCity.getMaintenance())

		elif (szKey == "TRADE"):
			# Trade status...
			nTotalTradeProfit = 0

			# For each trade route possible
			for nTradeRoute in range (gc.getDefineINT("MAX_TRADE_ROUTES")):
				# Get the next trade city
				pTradeCity = pCity.getTradeCity(nTradeRoute)
				# Not quite sure what this does but it's in the MainInterface
				# and I pretty much C&Ped :p
				if (pTradeCity and pTradeCity.getOwner() >= 0):
					for j in range( YieldTypes.NUM_YIELD_TYPES ):
						iTradeProfit = pCity.calculateTradeYield(j, pCity.calculateTradeProfit(pTradeCity))

						# If the TradeProfit is greater than 0 and it to the total
						if ( iTradeProfit > 0 ):
							nTotalTradeProfit += iTradeProfit

			szReturn = unicode(nTotalTradeProfit)

		elif (szKey == "COMMERCE"):
			# Commerce status...
			szReturn = unicode(pCity.calculateGoldRate())

		elif (szKey == "GOLD"):
			# Gold status...
			szReturn = unicode(pCity.reallyGold())

		elif (szKey == "RESEARCH"):
			# Science rate...
			szReturn = unicode(pCity.getResearchRate())

		elif (szKey == "CULTURE_RATE"):
			# Culture status...
			szReturn = unicode(pCity.getCultureCommerce())

		elif (szKey == "CULTURE"):
			# Total Culture
			szReturn = unicode(pCity.getCulture())

		elif (szKey == "GREATPEOPLE_RATE"):
			# Great Person
			szReturn = unicode(pCity.getGreatPeopleRate())

		elif (szKey == "GREATPEOPLE"):
			# Great Person
			szReturn = unicode(pCity.getGreatPeopleProgress())

		elif (szKey == "PRODUCING"):
			# Producing
			# If there's something in the queue,
			if (pCity.getOrderQueueLength() > 0):
				# Get the name of whatever it's producing.
				szReturn = pCity.getProductionName()
			# Otherwise we're not producing anything. Leave it blank.
			else:
				szReturn = "-"
			# Store what the city is producing
			self.CITY_PRODUCING_DICT[pCity.getName()] = szReturn

		elif (szKey == "TURNS"):
			# Turns left to finish actual production 
			# If there's something in the queue,
			if (pCity.getOrderQueueLength() > 0):
				szReturn =  unicode(pCity.getProductionTurnsLeft())
			# Otherwise we're not producing anything. Leave it blank.
			else:
				szReturn = "-"
			
		elif (szKey == "RELIGIONS"):
			# Religions...

			lHolyCity = pCity.getHolyCity()
			if lHolyCity:
				iI = 0
				for iI in range(len(lHolyCity)):
					szReturn += u"%c" %(gc.getReligionInfo(lHolyCity[iI]).getHolyCityChar())
			lReligions = pCity.getReligions()
			if lReligions:
				iI = 0
				for iI in range(len(lReligions)):
					if lReligions[iI] in lHolyCity:
						continue
					szReturn += u"%c" %(gc.getReligionInfo(lReligions[iI]).getChar())

		elif (szKey == "SPECIALISTS"):
			# Specialists...

			szReturn += "<font=1>"

			# For each specialist type
			for i in range( self.getNumSpecialistInfos() ):

#				for j in range (pCity.getSpecialistCount (i)):
#					szReturn += self.SPECIALIST_ICON_DICT[i]
				nCount = pCity.getSpecialistCount (i)
				# If more than one specialist
#				if (nCount > 1):
#					szReturn += self.SPECIALIST_ICON_DICT[i] + "x" + str(nCount) + " "
#				elif (nCount == 1):
#					szReturn += self.SPECIALIST_ICON_DICT[i] + " "
				szReturn += self.SPECIALIST_ICON_DICT[i]*nCount
				
			szReturn += "</font>"

		elif (szKey == "AUTOMATION"):
			# Automated values
			szReturn += "<font=1>"

			nNumEmphasize = self.getNumEmphasizeInfos()
			if pCity.isCitizensAutomated():
				szReturn += "C"
			if pCity.isProductionAutomated():
				szReturn += "P"
			for i in range( nNumEmphasize ):
				nNum = nNumEmphasize - i - 1
				if (pCity.AI_isEmphasize (nNum)):
					szReturn += self.AUTOMATION_ICON_DICT[nNum]

			szReturn += "</font>"

			# return the final value
		return szReturn


	def CheckCity ( self, pCity, i ):
		if pCity.hasBuilding(i): 
			return localText.changeTextColor ("+", gc.getInfoTypeForString("COLOR_GREEN")) #"+"
		if (pCity.getOrderQueueLength() > 0) & (gc.getBuildingInfo(i).getDescription() == pCity.getProductionName()):
			return u"%c" %(gc.getYieldInfo(YieldTypes.YIELD_PRODUCTION).getChar())
		if (isTeamWonderClass(gc.getBuildingInfo(i).getBuildingClassType())):
			if not pCity.canCreate(i):
				return localText.changeTextColor ("-", gc.getInfoTypeForString("COLOR_RED")) #"-"
		else:
			if not pCity.canConstruct(i):
				return localText.changeTextColor ("-", gc.getInfoTypeForString("COLOR_RED")) #"-"
		return ""
		
	def ColorCityValues (self, nValue, szKey):
		"Colors city values which we might want to alert the user to."
		# If the key is one we want to possibly color
		if (szKey in self.COLOR_SET):
			# If we don't have a plain integer
			if (re.search ("[-+]", nValue)):
				# Color it red and return it
				return localText.changeTextColor (nValue, gc.getInfoTypeForString("COLOR_RED"))
			# For each type of comparison
			for szCompareType, clDict in self.COLOR_DICT_DICT.items():
				# Get the color we will use.
				color = self.COLOR_DICT[szCompareType]
				# If the dictionary has the key and the comparison is appropriate
				if (clDict.has_key(szKey) and (szCompareType == "PROBLEM" and int(nValue) <= clDict[szKey] or szCompareType == "NEUTRAL" and int(nValue) == clDict[szKey] or szCompareType == "GREAT" and int(nValue) >= clDict[szKey])):
					# Color and return it
					return localText.changeTextColor (nValue, color)
		# Otherwise, just return the regular value
		return nValue

	# Function to draw the contents of the cityList passed in
	def drawContents (self, szArea):
		# Get the screen and the player
		screen = self.getScreen()
		iPlayer = PyPlayer(CyGame().getActivePlayer())
		screen.moveToFront( "Background" )
		if (szArea == self.DEFAULT_AREA_NAME):
			dDict = self.DEF_COLUMN_DICT
			screen.addTableControlGFC (szArea, len (dDict), self.nTableX, self.nTableY, self.nTableWidth, self.nTableLength, True, False, 32, 32, TableStyles.TABLE_STYLE_STANDARD )
		elif (szArea == self.SPECIAL_AREA_NAME):
			dDict = self.SPEC_COLUMN_DICT		
			screen.addTableControlGFC (szArea, len (dDict), self.nTableX, self.nTableY, self.nTableWidth, self.nTableLength, True, False, 32, 32, TableStyles.TABLE_STYLE_STANDARD )
		elif (szArea == self.BUILD_AREA_NAME):
			screen.addTableControlGFC (szArea, self.BUILD_TO_COL-self.BUILD_FROM_COL+3, self.nTableX, self.nTableY, self.nTableWidth, self.nTableLength, True, False, 32, 32, TableStyles.TABLE_STYLE_STANDARD )
		elif (szArea == self.WW_AREA_NAME):
			screen.addTableControlGFC (szArea, self.WW_TO_COL-self.WW_FROM_COL+3, self.nTableX, self.nTableY, self.nTableWidth, self.nTableLength, True, False, 32, 32, TableStyles.TABLE_STYLE_STANDARD )
		elif (szArea == self.NW_AREA_NAME):
			screen.addTableControlGFC (szArea, self.NW_TO_COL-self.NW_FROM_COL+3, self.nTableX, self.nTableY, self.nTableWidth, self.nTableLength, True, False, 32, 32, TableStyles.TABLE_STYLE_STANDARD )
		else:
			return
		# Build the table	
		screen.enableSelect (szArea, True )
		# Loop through the cities
		cityList = iPlayer.getCityList()
		for i in range(len(cityList)):
			pLoopCity = PyCity( CyGame().getActivePlayer(), cityList[i].getID() )
			screen.appendTableRow (szArea)
			if ((szArea == self.BUILD_AREA_NAME) or (szArea == self.WW_AREA_NAME) or (szArea == self.NW_AREA_NAME)):
				# date founded column
				szValue = self.ColorCityValues (self.CalculateValue (pLoopCity, "DATE"), "DATE")
				screen.setTableText (szArea, 0, i, szValue, "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)
				# city name column
				szValue = self.ColorCityValues (self.CalculateValue (pLoopCity, "NAME"), "NAME")
				screen.setTableText (szArea, 1, i, szValue, "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)
				# buildings column
				l = 1
				if (szArea == self.BUILD_AREA_NAME):
					lList = self.lBuildings
					iFrom = self.BUILD_FROM_COL
					iTo = self.BUILD_TO_COL
				elif (szArea == self.WW_AREA_NAME):
					lList = self.lWW
					iFrom = self.WW_FROM_COL
					iTo = self.WW_TO_COL
				elif (szArea == self.NW_AREA_NAME):
					lList = self.lNW	
					iFrom = self.NW_FROM_COL
					iTo = self.NW_TO_COL
				else:
					return
				for iI in range(len(lList)):
					if (iI >= iFrom) & ( iI <= iTo ):
						l += 1
						szValue = self.CheckCity( pLoopCity, lList[iI][0] )
						screen.setTableText (szArea, l, i, szValue, "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_CENTER_JUSTIFY)
			else:
				# For each column
				for key, value in dDict.items():
					justify = self.JUSTIFY_DICT[key]
					if (justify == CvUtil.FONT_RIGHT_JUSTIFY):
						funcTableWrite = screen.setTableInt
					else:
						funcTableWrite = screen.setTableText
					szValue = self.ColorCityValues (self.CalculateValue (pLoopCity, key), key)
					funcTableWrite (szArea, value, i, szValue, "", WidgetTypes.WIDGET_GENERAL, -1, -1, justify)
		self.drawHeaders (szArea)
		if (self.m_nChosenCity < len(cityList) and \
				self.m_nChosenCity != self.NO_CITY):
			screen.selectRow (szArea, self.m_nChosenCity, True)
		else:
			self.m_nChosenCity = self.NO_CITY
		screen.moveToBack (self.BACKGROUND_ID)
		self.updateAppropriateCitySelection (szArea, len( PyPlayer(CyGame().getActivePlayer()).getCityList() ) )
		
	def HandleSpecialistPlus (self, inputClass):
		pCity = self.getCurrentCity()
		if (pCity):
			if self.m_bSpecialistChange and self.m_nLastSpecialistChange == inputClass.getID():
				nMaxSpecial = pCity.getMaxSpecialistCount (self.m_nLastSpecialistChange)
				self.m_nSpecialistChangeNumber += 1
				if (self.m_nSpecialistChangeNumber > nMaxSpecial):
					self.m_nSpecialistChangeNumber = nMaxSpecial
			else:
				self.m_bSpecialistChange = True
				self.m_nLastSpecialistChange = inputClass.getID()
				self.m_nSpecialistChangeNumber = pCity.getSpecialistCount(self.m_nLastSpecialistChange) + 1
		return 0

		
	def HandleSpecialistMinus (self, inputClass):
		pCity = self.getCurrentCity()
		if (pCity):
			if self.m_bSpecialistChange and self.m_nLastSpecialistChange == inputClass.getID():
				self.m_nSpecialistChangeNumber -= 1
				if (self.m_nSpecialistChangeNumber < 0):
					self.m_nSpecialistChangeNumber = 0
			else:
				self.m_bSpecialistChange = True
				self.m_nLastSpecialistChange = inputClass.getID()
				self.m_nSpecialistChangeNumber = pCity.getSpecialistCount(self.m_nLastSpecialistChange) - 1
#			self.m_nTimesChanged = 0
		return 0

	def Spec (self, inputClass):
		self.m_szMode = self.SPECIAL_AREA_NAME
		self.drawScreen (self.m_szMode)
		return 1
		
	def City (self, inputClass):
		self.m_szMode = self.DEFAULT_AREA_NAME
		self.drawScreen (self.m_szMode)
		return 1
				
			
	def DomesticExit (self, inputClass):
		return 0


	def View (self, inputClass):
		screen = self.getScreen()
		self.hide (screen, self.m_szMode)
		screen.hideScreen()	
		# Grab the city indicated by the index
		city = CyGlobalContext().getActivePlayer().getCity(self.m_nChosenCity)
		CyInterface().selectCity(city, True)
		# And jump straight to it
		screen = CyGInterfaceScreen( "DomesticAdvisor", CvScreenEnums.DOMESTIC_ADVISOR )
		screen.hideScreen()
		return 1

	def Redraw (self, inputClass):
		# Redraw the screen
		self.drawScreen (self.m_szMode)
		return 1

	def Build (self, inputClass):
		self.m_szMode = self.BUILD_AREA_NAME
		self.drawScreen (self.m_szMode)
		return 1

	def WW (self, inputClass):
		self.m_szMode = self.WW_AREA_NAME
		self.drawScreen (self.m_szMode)
		return 1

	def NW (self, inputClass):
		self.m_szMode = self.NW_AREA_NAME
		self.drawScreen (self.m_szMode)
		return 1

	def BuildLeft (self, inputClass):
		if (self.m_szMode == self.BUILD_AREA_NAME):
			if (self.BUILD_FROM_COL > 0):
				self.BUILD_FROM_COL -= self.BUILD_NUM_COLS
				self.BUILD_TO_COL -= self.BUILD_NUM_COLS
				if self.BUILD_FROM_COL < 0:
					self.BUILD_FROM_COL = 0
					self.BUILD_TO_COL = self.BUILD_NUM_COLS-1
				self.drawScreen (self.m_szMode)
		elif (self.m_szMode == self.WW_AREA_NAME):
			if (self.WW_FROM_COL > 0):
				self.WW_FROM_COL -= self.BUILD_NUM_COLS
				self.WW_TO_COL -= self.BUILD_NUM_COLS
				if self.WW_FROM_COL < 0:
					self.WW_FROM_COL = 0
					self.WW_TO_COL = self.BUILD_NUM_COLS-1
				self.drawScreen (self.m_szMode)
		elif (self.m_szMode == self.NW_AREA_NAME):
			if (self.NW_FROM_COL > 0):
				self.NW_FROM_COL -= self.BUILD_NUM_COLS
				self.NW_TO_COL -= self.BUILD_NUM_COLS
				if self.NW_FROM_COL < 0:
					self.NW_FROM_COL = 0
					self.NW_TO_COL = self.BUILD_NUM_COLS-1
				self.drawScreen (self.m_szMode)
		return 1
		
	def BuildRight (self, inputClass):
		if (self.m_szMode == self.BUILD_AREA_NAME):
			if (self.BUILD_TO_COL < (len(self.lBuildings)-1)):
				self.BUILD_FROM_COL += self.BUILD_NUM_COLS
				self.BUILD_TO_COL += self.BUILD_NUM_COLS
				if self.BUILD_TO_COL > (len(self.lBuildings)-1):
					self.BUILD_TO_COL = len(self.lBuildings)-1
					self.BUILD_FROM_COL = len(self.lBuildings)-self.BUILD_NUM_COLS
				self.drawScreen (self.m_szMode)
		elif (self.m_szMode == self.WW_AREA_NAME):
			if (self.WW_TO_COL < (len(self.lWW)-1)):
				self.WW_FROM_COL += self.BUILD_NUM_COLS
				self.WW_TO_COL += self.BUILD_NUM_COLS
				if self.WW_TO_COL > (len(self.lWW)-1):
					self.WW_TO_COL = len(self.lWW)-1
					self.WW_FROM_COL = len(self.lWW)-self.BUILD_NUM_COLS
				self.drawScreen (self.m_szMode)
		elif (self.m_szMode == self.NW_AREA_NAME):
			if (self.NW_TO_COL < (len(self.lNW)-1)):
				self.NW_FROM_COL += self.BUILD_NUM_COLS
				self.NW_TO_COL += self.BUILD_NUM_COLS
				if self.NW_TO_COL > (len(self.lNW)-1):
					self.NW_TO_COL = len(self.lNW)-1
					self.NW_FROM_COL = len(self.lNW)-self.BUILD_NUM_COLS
				self.drawScreen (self.m_szMode)
		return 1
		
	# Will handle the input for this screen...
	def handleInput (self, inputClass):
		' Calls function mapped in DomesticAdvisorInputMap'
		# only get from the map if it has the key
		# If we get that a listbox item was selected
		if ( inputClass.getNotifyCode() == NotifyCode.NOTIFY_LISTBOX_ITEM_SELECTED ):
			screen = self.getScreen()

			# Store the index
			nIndex = inputClass.getData()
			self.m_nChosenCity = nIndex

			if (self.m_szMode == self.SPECIAL_AREA_NAME):
				self.showSpecialists()

			# And pass it back to the screen
			self.updateAppropriateCitySelection( self.m_szMode, len( PyPlayer(CyGame().getActivePlayer()).getCityList() ) )

			# TODO: Note, there is NO possible way we can tell which city is actually
			# being indicated. As such, we don't want to jump to a city which might
			# not be the correct one. Because of this we just pass it to the screen
			# who DOES have the information to determine which city it is.

			# Perhaps in the future this will be opened up to us so that we can make
			# the call from a higher level.

			# Because of this problem, I had to add a redraw function which sets the
			# list back to the known order so that the jump can't be correctly made.
			
		elif (inputClass.getNotifyCode() == NotifyCode.NOTIFY_CHARACTER):
			if (inputClass.getData() == int(InputTypes.KB_V)):
				return self.View (inputClass)
			elif (inputClass.getData() == int(InputTypes.KB_R)):
				return self.Redraw (inputClass)				
			elif (inputClass.getData() == int(InputTypes.KB_C)):
				return self.City (inputClass)
			elif (inputClass.getData() == int(InputTypes.KB_S)):
				return self.Spec (inputClass)

		# Otherwise, we got something else
		elif (inputClass.getNotifyCode() == NotifyCode.NOTIFY_CLICKED):
			if (self.DomesticScreenInputMap.has_key(inputClass.getFunctionName())):
				'Calls function mapped in CvSpecDomesticAdvisor'
				# only get from the map if it has the key		
				# get bound function from map and call it
				return self.DomesticScreenInputMap.get(inputClass.getFunctionName())(inputClass)
		return 0

	def updateAppropriateCitySelection(self, scr, nCities):
		screen = CyGInterfaceScreen( "DomesticAdvisor", CvScreenEnums.DOMESTIC_ADVISOR )
		screen.updateAppropriateCitySelection( scr, nCities, 1 )
		
	def isFoodProduction (self, szProducing):
		return re.search ("Worker|Settler", szProducing)

	def isFoodProductionChange (self, pCity):
		szCity = pCity.getName()
		# If we already have noted what this city was producing
		if (self.CITY_PRODUCING_DICT.has_key (szCity)):
			# Get if we switched to/from a food production
			bFoodProductionSwitch = ((pCity.isFoodProduction () and \
									 not (self.isFoodProduction (self.CITY_PRODUCING_DICT[szCity]))) or \
									(not (pCity.isFoodProduction ()) and \
									self.isFoodProduction (self.CITY_PRODUCING_DICT[szCity])))
		else:
			# Otherwise, there's no way it's a food production switch
			bFoodProductionSwitch = False
		return bFoodProductionSwitch

	def updateScreen(self):
		screen = self.getScreen()
		iPlayer = PyPlayer(CyGame().getActivePlayer())
		if (self.m_szMode == self.DEFAULT_AREA_NAME):
			dDict = self.DEF_COLUMN_DICT
		elif (self.m_szMode == self.SPECIAL_AREA_NAME):
			dDict = self.SPEC_COLUMN_DICT		
		elif ((self.m_szMode == self.BUILD_AREA_NAME) or (self.m_szMode == self.WW_AREA_NAME) or (self.m_szMode == self.NW_AREA_NAME)):
			self.drawScreen (self.m_szMode)
			return
		else:
			return		
		cityList = iPlayer.getCityList()
		# For each city
		for i in range(len(cityList)):
			pLoopCity = PyCity( CyGame().getActivePlayer(), cityList[i].getID() )
			bFoodProductionChange = self.isFoodProductionChange (pLoopCity)
			# For each value that needs updating
			for key, value in self.UPDATE_DICT.items():
				# If the key is in this table and it's updatable with the change type
				if dDict.has_key (key) and (bFoodProductionChange or value):
					# Update it
					szValue = self.ColorCityValues (self.CalculateValue (pLoopCity, key), key)
					screen.setTableTextKey (self.m_szMode, dDict[key], pLoopCity.getName(), 1, szValue, WidgetTypes.WIDGET_GENERAL, -1, -1, self.JUSTIFY_DICT[key], len(cityList))
		return

	def handleSpecialistUpdate (self):
		""" This function returns if we should handle the Specialist Update."""
		# If we have a specialist change and it's in the specialist table mode
		if (self.m_bSpecialistChange and (self.m_szMode == self.SPECIAL_AREA_NAME)):
			# Get the current city and return if the specialist has completed changing
			pCity = self.getCurrentCity()
			if (pCity):
				return self.isSpecialistChange(pCity)
		return False

	def isSpecialistChange (self, pCity):
		return self.m_nSpecialistChangeNumber == pCity.getSpecialistCount(self.m_nLastSpecialistChange)

	# Function to draw the contents of the cityList passed in
	def updateRow (self, szArea, nRow):

		# Get the screen and the player
		screen = self.getScreen()

		if (szArea == self.SPECIAL_AREA_NAME):
			dDict = self.SPEC_COLUMN_DICT
		else:
			return

		iPlayer = PyPlayer(CyGame().getActivePlayer())
		cityList = iPlayer.getCityList()

		pCity = PyCity( CyGame().getActivePlayer(), cityList[nRow].getID() )

		# For each column
		for key, value in dDict.items():
			justify = self.JUSTIFY_DICT[key]
			if (justify == CvUtil.FONT_LEFT_JUSTIFY):
				funcTableWrite = screen.setTableText
			else:
				funcTableWrite = screen.setTableInt

			szValue = self.ColorCityValues (self.CalculateValue (pCity, key), key)
			funcTableWrite (szArea, value, nRow, szValue, "", WidgetTypes.WIDGET_GENERAL, -1, -1, justify)

		self.updateAppropriateCitySelection (szArea, len( PyPlayer(CyGame().getActivePlayer()).getCityList() ) )

#	def updatewithMainInterface (self, fDelta):
#		return
		
	def update(self, fDelta):
		""" Update the Advisor."""
#		SpecDomPrint ("Entering update")
		# If we can handle the specialist update, do so.
		if (self.handleSpecialistUpdate()):
			self.showSpecialists()
			self.updateRow (self.m_szMode, self.m_nChosenCity)
			# We've handled it.
			self.m_bSpecialistChange = False

		return
